@@ -1,22 +1,48 @@
 """WebSocket client for asyncio."""
 
 import asyncio
-import json
+from typing import Any, Optional, cast
 
 import async_timeout
 
 from .client_exceptions import ClientError
+from .client_reqrep import ClientResponse
 from .helpers import call_later, set_result
-from .http import (WS_CLOSED_MESSAGE, WS_CLOSING_MESSAGE, WebSocketError,
-                   WSMessage, WSMsgType)
+from .http import (
+    WS_CLOSED_MESSAGE,
+    WS_CLOSING_MESSAGE,
+    WebSocketError,
+    WSCloseCode,
+    WSMessage,
+    WSMsgType,
+)
+from .http_websocket import WebSocketWriter  # WSMessage
+from .streams import EofStream, FlowControlDataQueue
+from .typedefs import (
+    DEFAULT_JSON_DECODER,
+    DEFAULT_JSON_ENCODER,
+    JSONDecoder,
+    JSONEncoder,
+)
 
 
 class ClientWebSocketResponse:
-
-    def __init__(self, reader, writer, protocol,
-                 response, timeout, autoclose, autoping, loop, *,
-                 receive_timeout=None, heartbeat=None,
-                 compress=0, client_notakeover=False):
+    def __init__(
+        self,
+        reader: "FlowControlDataQueue[WSMessage]",
+        writer: WebSocketWriter,
+        protocol: Optional[str],
+        response: ClientResponse,
+        timeout: float,
+        autoclose: bool,
+        autoping: bool,
+        loop: asyncio.AbstractEventLoop,
+        *,
+        receive_timeout: Optional[float] = None,
+        heartbeat: Optional[float] = None,
+        compress: int = 0,
+        client_notakeover: bool = False,
+    ) -> None:
         self._response = response
         self._conn = response.connection
 
@@ -25,25 +51,25 @@ class ClientWebSocketResponse:
         self._protocol = protocol
         self._closed = False
         self._closing = False
-        self._close_code = None
+        self._close_code = None  # type: Optional[int]
         self._timeout = timeout
         self._receive_timeout = receive_timeout
         self._autoclose = autoclose
         self._autoping = autoping
         self._heartbeat = heartbeat
-        self._heartbeat_cb = None
+        self._heartbeat_cb: Optional[asyncio.TimerHandle] = None
         if heartbeat is not None:
-            self._pong_heartbeat = heartbeat/2.0
-        self._pong_response_cb = None
+            self._pong_heartbeat = heartbeat / 2.0
+        self._pong_response_cb: Optional[asyncio.TimerHandle] = None
         self._loop = loop
-        self._waiting = None
-        self._exception = None
+        self._waiting = None  # type: Optional[asyncio.Future[bool]]
+        self._exception = None  # type: Optional[BaseException]
         self._compress = compress
         self._client_notakeover = client_notakeover
 
         self._reset_heartbeat()
 
-    def _cancel_heartbeat(self):
+    def _cancel_heartbeat(self) -> None:
         if self._pong_response_cb is not None:
             self._pong_response_cb.cancel()
             self._pong_response_cb = None
@@ -52,81 +78,93 @@ class ClientWebSocketResponse:
             self._heartbeat_cb.cancel()
             self._heartbeat_cb = None
 
-    def _reset_heartbeat(self):
+    def _reset_heartbeat(self) -> None:
         self._cancel_heartbeat()
 
         if self._heartbeat is not None:
             self._heartbeat_cb = call_later(
-                self._send_heartbeat, self._heartbeat, self._loop)
+                self._send_heartbeat, self._heartbeat, self._loop
+            )
 
-    def _send_heartbeat(self):
+    def _send_heartbeat(self) -> None:
         if self._heartbeat is not None and not self._closed:
-            self._writer.ping()
+            # fire-and-forget a task is not perfect but maybe ok for
+            # sending ping. Otherwise we need a long-living heartbeat
+            # task in the class.
+            self._loop.create_task(self._writer.ping())
 
             if self._pong_response_cb is not None:
                 self._pong_response_cb.cancel()
             self._pong_response_cb = call_later(
-                self._pong_not_received, self._pong_heartbeat, self._loop)
+                self._pong_not_received, self._pong_heartbeat, self._loop
+            )
 
-    def _pong_not_received(self):
+    def _pong_not_received(self) -> None:
         if not self._closed:
             self._closed = True
-            self._close_code = 1006
+            self._close_code = WSCloseCode.ABNORMAL_CLOSURE
             self._exception = asyncio.TimeoutError()
             self._response.close()
 
     @property
-    def closed(self):
+    def closed(self) -> bool:
         return self._closed
 
     @property
-    def close_code(self):
+    def close_code(self) -> Optional[int]:
         return self._close_code
 
     @property
-    def protocol(self):
+    def protocol(self) -> Optional[str]:
         return self._protocol
 
     @property
-    def compress(self):
+    def compress(self) -> int:
         return self._compress
 
     @property
-    def client_notakeover(self):
+    def client_notakeover(self) -> bool:
         return self._client_notakeover
 
-    def get_extra_info(self, name, default=None):
+    def get_extra_info(self, name: str, default: Any = None) -> Any:
         """extra info from connection transport"""
-        try:
-            return self._response.connection.transport.get_extra_info(
-                name, default)
-        except Exception:
+        conn = self._response.connection
+        if conn is None:
             return default
+        transport = conn.transport
+        if transport is None:
+            return default
+        return transport.get_extra_info(name, default)
 
-    def exception(self):
+    def exception(self) -> Optional[BaseException]:
         return self._exception
 
-    async def ping(self, message='b'):
+    async def ping(self, message: bytes = b"") -> None:
         await self._writer.ping(message)
 
-    async def pong(self, message='b'):
+    async def pong(self, message: bytes = b"") -> None:
         await self._writer.pong(message)
 
-    async def send_str(self, data, compress=None):
+    async def send_str(self, data: str, compress: Optional[int] = None) -> None:
         if not isinstance(data, str):
-            raise TypeError('data argument must be str (%r)' % type(data))
+            raise TypeError("data argument must be str (%r)" % type(data))
         await self._writer.send(data, binary=False, compress=compress)
 
-    async def send_bytes(self, data, compress=None):
+    async def send_bytes(self, data: bytes, compress: Optional[int] = None) -> None:
         if not isinstance(data, (bytes, bytearray, memoryview)):
-            raise TypeError('data argument must be byte-ish (%r)' %
-                            type(data))
+            raise TypeError("data argument must be byte-ish (%r)" % type(data))
         await self._writer.send(data, binary=True, compress=compress)
 
-    async def send_json(self, data, compress=None, *, dumps=json.dumps):
+    async def send_json(
+        self,
+        data: Any,
+        compress: Optional[int] = None,
+        *,
+        dumps: JSONEncoder = DEFAULT_JSON_ENCODER,
+    ) -> None:
         await self.send_str(dumps(data), compress=compress)
 
-    async def close(self, *, code=1000, message=b''):
+    async def close(self, *, code: int = WSCloseCode.OK, message: bytes = b"") -> bool:
         # we need to break `receive()` cycle first,
         # `close()` may be called from different task
         if self._waiting is not None and not self._closed:
@@ -137,13 +175,13 @@ class ClientWebSocketResponse:
             self._cancel_heartbeat()
             self._closed = True
             try:
-                self._writer.close(code, message)
+                await self._writer.close(code, message)
             except asyncio.CancelledError:
-                self._close_code = 1006
+                self._close_code = WSCloseCode.ABNORMAL_CLOSURE
                 self._response.close()
                 raise
             except Exception as exc:
-                self._close_code = 1006
+                self._close_code = WSCloseCode.ABNORMAL_CLOSURE
                 self._exception = exc
                 self._response.close()
                 return True
@@ -154,14 +192,14 @@ class ClientWebSocketResponse:
 
             while True:
                 try:
-                    with async_timeout.timeout(self._timeout, loop=self._loop):
+                    async with async_timeout.timeout(self._timeout):
                         msg = await self._reader.read()
                 except asyncio.CancelledError:
-                    self._close_code = 1006
+                    self._close_code = WSCloseCode.ABNORMAL_CLOSURE
                     self._response.close()
                     raise
                 except Exception as exc:
-                    self._close_code = 1006
+                    self._close_code = WSCloseCode.ABNORMAL_CLOSURE
                     self._exception = exc
                     self._response.close()
                     return True
@@ -173,11 +211,10 @@ class ClientWebSocketResponse:
         else:
             return False
 
-    async def receive(self, timeout=None):
+    async def receive(self, timeout: Optional[float] = None) -> WSMessage:
         while True:
             if self._waiting is not None:
-                raise RuntimeError(
-                    'Concurrent call to receive() is not allowed')
+                raise RuntimeError("Concurrent call to receive() is not allowed")
 
             if self._closed:
                 return WS_CLOSED_MESSAGE
@@ -188,9 +225,7 @@ class ClientWebSocketResponse:
             try:
                 self._waiting = self._loop.create_future()
                 try:
-                    with async_timeout.timeout(
-                            timeout or self._receive_timeout,
-                            loop=self._loop):
+                    async with async_timeout.timeout(timeout or self._receive_timeout):
                         msg = await self._reader.read()
                     self._reset_heartbeat()
                 finally:
@@ -198,11 +233,15 @@ class ClientWebSocketResponse:
                     self._waiting = None
                     set_result(waiter, True)
             except (asyncio.CancelledError, asyncio.TimeoutError):
-                self._close_code = 1006
+                self._close_code = WSCloseCode.ABNORMAL_CLOSURE
                 raise
+            except EofStream:
+                self._close_code = WSCloseCode.OK
+                await self.close()
+                return WSMessage(WSMsgType.CLOSED, None, None)
             except ClientError:
                 self._closed = True
-                self._close_code = 1006
+                self._close_code = WSCloseCode.ABNORMAL_CLOSURE
                 return WS_CLOSED_MESSAGE
             except WebSocketError as exc:
                 self._close_code = exc.code
@@ -211,7 +250,7 @@ class ClientWebSocketResponse:
             except Exception as exc:
                 self._exception = exc
                 self._closing = True
-                self._close_code = 1006
+                self._close_code = WSCloseCode.ABNORMAL_CLOSURE
                 await self.close()
                 return WSMessage(WSMsgType.ERROR, exc, None)
 
@@ -230,33 +269,32 @@ class ClientWebSocketResponse:
 
             return msg
 
-    async def receive_str(self, *, timeout=None):
+    async def receive_str(self, *, timeout: Optional[float] = None) -> str:
         msg = await self.receive(timeout)
         if msg.type != WSMsgType.TEXT:
-            raise TypeError(
-                "Received message {}:{!r} is not str".format(msg.type,
-                                                             msg.data))
-        return msg.data
+            raise TypeError(f"Received message {msg.type}:{msg.data!r} is not str")
+        return cast(str, msg.data)
 
-    async def receive_bytes(self, *, timeout=None):
+    async def receive_bytes(self, *, timeout: Optional[float] = None) -> bytes:
         msg = await self.receive(timeout)
         if msg.type != WSMsgType.BINARY:
-            raise TypeError(
-                "Received message {}:{!r} is not bytes".format(msg.type,
-                                                               msg.data))
-        return msg.data
+            raise TypeError(f"Received message {msg.type}:{msg.data!r} is not bytes")
+        return cast(bytes, msg.data)
 
-    async def receive_json(self, *, loads=json.loads, timeout=None):
+    async def receive_json(
+        self,
+        *,
+        loads: JSONDecoder = DEFAULT_JSON_DECODER,
+        timeout: Optional[float] = None,
+    ) -> Any:
         data = await self.receive_str(timeout=timeout)
         return loads(data)
 
-    def __aiter__(self):
+    def __aiter__(self) -> "ClientWebSocketResponse":
         return self
 
-    async def __anext__(self):
+    async def __anext__(self) -> WSMessage:
         msg = await self.receive()
-        if msg.type in (WSMsgType.CLOSE,
-                        WSMsgType.CLOSING,
-                        WSMsgType.CLOSED):
-            raise StopAsyncIteration  # NOQA
+        if msg.type in (WSMsgType.CLOSE, WSMsgType.CLOSING, WSMsgType.CLOSED):
+            raise StopAsyncIteration
         return msg
