@@ -1,28 +1,24 @@
 import asyncio
 import collections
-from typing import Any, Optional
-
-
-try:
-    from typing import Deque
-except ImportError:
-    from typing_extensions import Deque  # noqa
+from typing import Any, Deque, Optional
 
 
 class EventResultOrError:
-    """
-    This class wrappers the Event asyncio lock allowing either awake the
+    """Event asyncio lock helper class.
+
+    Wraps the Event asyncio lock allowing either to awake the
     locked Tasks without any error or raising an exception.
 
     thanks to @vorpalsmith for the simple design.
     """
+
     def __init__(self, loop: asyncio.AbstractEventLoop) -> None:
         self._loop = loop
         self._exc = None  # type: Optional[BaseException]
-        self._event = asyncio.Event(loop=loop)
+        self._event = asyncio.Event()
         self._waiters = collections.deque()  # type: Deque[asyncio.Future[Any]]
 
-    def set(self, exc: Optional[BaseException]=None) -> None:
+    def set(self, exc: Optional[BaseException] = None) -> None:
         self._exc = exc
         self._event.set()
 
@@ -40,6 +36,6 @@ class EventResultOrError:
         return val
 
     def cancel(self) -> None:
-        """ Cancel all waiters """
+        """Cancel all waiters"""
         for waiter in self._waiters:
             waiter.cancel()
