@@ -36,7 +36,9 @@ def test_register_unsupported_order(registry) -> None:
         pass
 
     with pytest.raises(ValueError):
-        payload.register_payload(Payload, TestProvider, order=object())
+        payload.register_payload(
+            Payload, TestProvider, order=object()  # type: ignore[arg-type]
+        )
 
 
 def test_payload_ctor() -> None:
@@ -65,7 +67,7 @@ def test_bytes_payload_explicit_content_type() -> None:
 
 def test_bytes_payload_bad_type() -> None:
     with pytest.raises(TypeError):
-        payload.BytesPayload(object())
+        payload.BytesPayload(object())  # type: ignore[arg-type]
 
 
 def test_bytes_payload_memoryview_correct_size() -> None:
