@@ -5,11 +5,12 @@ import ssl
 import unittest
 from unittest import mock
 
+import pytest
 from yarl import URL
 
 import aiohttp
 from aiohttp.client_reqrep import ClientRequest, ClientResponse
-from aiohttp.helpers import TimerNoop
+from aiohttp.helpers import PY_37, TimerNoop
 from aiohttp.test_utils import make_mocked_coro
 
 
@@ -72,6 +73,8 @@ class TestProxy(unittest.TestCase):
             ssl=None,
         )
 
+        conn.close()
+
     @mock.patch("aiohttp.connector.ClientRequest")
     def test_proxy_headers(self, ClientRequestMock) -> None:
         req = ClientRequest(
@@ -112,6 +115,8 @@ class TestProxy(unittest.TestCase):
             ssl=None,
         )
 
+        conn.close()
+
     def test_proxy_auth(self) -> None:
         with self.assertRaises(ValueError) as ctx:
             ClientRequest(
@@ -221,6 +226,7 @@ class TestProxy(unittest.TestCase):
 
         tr, proto = mock.Mock(), mock.Mock()
         self.loop.create_connection = make_mocked_coro((tr, proto))
+        self.loop.start_tls = make_mocked_coro(mock.Mock())
 
         req = ClientRequest(
             "GET",
@@ -235,8 +241,6 @@ class TestProxy(unittest.TestCase):
         self.assertEqual(req.url.path, "/")
         self.assertEqual(proxy_req.method, "CONNECT")
         self.assertEqual(proxy_req.url, URL("https://www.python.org"))
-        tr.close.assert_called_once_with()
-        tr.get_extra_info.assert_called_with("socket", default=None)
 
         self.loop.run_until_complete(proxy_req.close())
         proxy_resp.close()
@@ -280,22 +284,10 @@ class TestProxy(unittest.TestCase):
             ]
         )
 
-        seq = 0
-
-        async def create_connection(*args, **kwargs):
-            nonlocal seq
-            seq += 1
-
-            # connection to http://proxy.example.com
-            if seq == 1:
-                return mock.Mock(), mock.Mock()
-            # connection to https://www.python.org
-            elif seq == 2:
-                raise ssl.CertificateError
-            else:
-                assert False
-
-        self.loop.create_connection = create_connection
+        # Called on connection to http://proxy.example.com
+        self.loop.create_connection = make_mocked_coro((mock.Mock(), mock.Mock()))
+        # Called on connection to https://www.python.org
+        self.loop.start_tls = make_mocked_coro(raise_exception=ssl.CertificateError)
 
         req = ClientRequest(
             "GET",
@@ -346,22 +338,12 @@ class TestProxy(unittest.TestCase):
             ]
         )
 
-        seq = 0
-
-        async def create_connection(*args, **kwargs):
-            nonlocal seq
-            seq += 1
-
-            # connection to http://proxy.example.com
-            if seq == 1:
-                return mock.Mock(), mock.Mock()
-            # connection to https://www.python.org
-            elif seq == 2:
-                raise ssl.SSLError
-            else:
-                assert False
-
-        self.loop.create_connection = create_connection
+        # Called on connection to http://proxy.example.com
+        self.loop.create_connection = make_mocked_coro(
+            (mock.Mock(), mock.Mock()),
+        )
+        # Called on connection to https://www.python.org
+        self.loop.start_tls = make_mocked_coro(raise_exception=ssl.SSLError)
 
         req = ClientRequest(
             "GET",
@@ -374,6 +356,11 @@ class TestProxy(unittest.TestCase):
                 connector._create_connection(req, None, aiohttp.ClientTimeout())
             )
 
+    @pytest.mark.skipif(
+        PY_37,
+        reason="The tested code path is only reachable below Python 3.7 because those "
+        "versions don't yet have `asyncio.loop.start_tls()` implemeneted",
+    )
     @mock.patch("aiohttp.connector.ClientRequest")
     def test_https_connect_runtime_error(self, ClientRequestMock) -> None:
         proxy_req = ClientRequest(
@@ -643,6 +630,7 @@ class TestProxy(unittest.TestCase):
 
         tr, proto = mock.Mock(), mock.Mock()
         self.loop.create_connection = make_mocked_coro((tr, proto))
+        self.loop.start_tls = make_mocked_coro(mock.Mock())
 
         req = ClientRequest(
             "GET",
@@ -654,18 +642,17 @@ class TestProxy(unittest.TestCase):
             connector._create_connection(req, None, aiohttp.ClientTimeout())
         )
 
-        self.loop.create_connection.assert_called_with(
+        self.loop.start_tls.assert_called_with(
             mock.ANY,
-            ssl=connector._make_ssl_context(True),
-            sock=mock.ANY,
+            mock.ANY,
+            connector._make_ssl_context(True),
             server_hostname="www.python.org",
+            ssl_handshake_timeout=mock.ANY,
         )
 
         self.assertEqual(req.url.path, "/")
         self.assertEqual(proxy_req.method, "CONNECT")
         self.assertEqual(proxy_req.url, URL("https://www.python.org"))
-        tr.close.assert_called_once_with()
-        tr.get_extra_info.assert_called_with("socket", default=None)
 
         self.loop.run_until_complete(proxy_req.close())
         proxy_resp.close()
@@ -714,6 +701,7 @@ class TestProxy(unittest.TestCase):
 
         tr, proto = mock.Mock(), mock.Mock()
         self.loop.create_connection = make_mocked_coro((tr, proto))
+        self.loop.start_tls = make_mocked_coro(mock.Mock())
 
         self.assertIn("AUTHORIZATION", proxy_req.headers)
         self.assertNotIn("PROXY-AUTHORIZATION", proxy_req.headers)
